@@ -12,6 +12,8 @@ import re
 import numpy as np
 import logging
 
+from transformer_lens.torchtyping_helper import T
+
 
 class ActivationCache:
     """
@@ -116,6 +118,9 @@ class ActivationCache:
     def __iter__(self):
         return self.cache_dict.__iter__()
 
+    def __len__(self):
+        return len(self.cache_dict)
+
     def apply_slice_to_batch_dim(self, batch_slice: Union[Slice, SliceInput]):
         if not isinstance(batch_slice, Slice):
             batch_slice = Slice(batch_slice)
@@ -138,7 +143,7 @@ class ActivationCache:
         pos_slice: Union[Slice, SliceInput] = None,
         mlp_input: bool = False,
         return_labels: bool = False,
-    ) -> TT["layers_covered", "batch_and_pos_dims":..., "d_model"]:
+    ) -> TT[T.layers_covered, T.batch_and_pos_dims:..., T.d_model]:
         """Returns the accumulated residual stream up to a given layer, ie a stack of previous residual streams up to that layer's input. This can be thought of as a series of partial values of the residual stream, where the model gradually accumulates what it wants.
 
         Args:
@@ -185,7 +190,7 @@ class ActivationCache:
         pos_slice: Union[Slice, SliceInput] = None,
         incl_embeds: bool = True,
         return_labels: bool = False,
-    ) -> TT["layers_covered", "batch_and_pos_dims":..., "d_model"]:
+    ) -> TT[T.layers_covered, T.batch_and_pos_dims:..., T.d_model]:
         """Decomposes the residual stream input to layer L into a stack of the output of previous layers. The sum of these is the input to layer L (plus embedding and pos embedding). This is useful for attributing model behaviour to different components of the residual stream
 
         Args:
@@ -262,17 +267,17 @@ class ActivationCache:
 
     def stack_head_results(
         self,
-        layer: int,
+        layer: int = -1,
         return_labels: bool = False,
         incl_remainder: bool = False,
         pos_slice: Union[Slice, SliceInput] = None,
-    ) -> TT["num_components", "batch_and_pos_dims":..., "d_model"]:
-        """Returns a stack of all head results (ie residual stream contribution) up to layer L. A good way to decompose the outputs of attention layers into attribution by specific heads.
+    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
+        """Returns a stack of all head results (ie residual stream contribution) up to layer L. A good way to decompose the outputs of attention layers into attribution by specific heads. Note that the num_components axis has length layer x n_heads ((layer head_index) in einops notation)
 
         Assumes that the model has been run with use_attn_results=True
 
         Args:
-            layer (int): Layer index - heads at all layers strictly before this are included. layer must be in [1, n_layers]
+            layer (int): Layer index - heads at all layers strictly before this are included. layer must be in [1, n_layers-1], or any of (n_layers, -1, None), which all mean the final layer
             return_labels (bool, optional): Whether to also return a list of labels of the form "L0H0" for the heads. Defaults to False.
             incl_remainder (bool, optional): Whether to return a final term which is "the rest of the residual stream". Defaults to False.
             pos_slice (Slice): A slice object to apply to the pos dimension. Defaults to None, do nothing.
@@ -323,13 +328,38 @@ class ActivationCache:
             return components, labels
         else:
             return components
+    
+    def stack_activation(
+        self,
+        activation_name: str,
+        layer: int = -1,
+        sublayer_type: Optional[str] = None,
+    ) -> TT[T.layers_covered, ...]:
+        """Returns a stack of all head results (ie residual stream contribution) up to layer L. A good way to decompose the outputs of attention layers into attribution by specific heads. The output shape is exactly the same shape as the activations, just with a leading layers dimension.
+
+        Args:
+            activation_name (str): The name of the activation to be stacked
+            layer (int): Layer index - heads at all layers strictly before this are included. layer must be in [1, n_layers-1], or any of (n_layers, -1, None), which all mean the final layer
+            sublayer_type (str, *optional*): The sub layer type of the activation, passed to utils.get_act_name. Can normally be inferred
+            incl_remainder (bool, optional): Whether to return a final term which is "the rest of the residual stream". Defaults to False.
+        """
+        
+        if layer is None or layer == -1:
+            # Default to the residual stream immediately pre unembed
+            layer = self.model.cfg.n_layers
+        
+        components = []
+        for l in range(layer):
+            components.append(self[(activation_name, l, sublayer_type)])
+        
+        return torch.stack(components, dim=0)
 
     def get_neuron_results(
         self,
         layer: int,
         neuron_slice: Union[Slice, SliceInput] = None,
         pos_slice: Union[Slice, SliceInput] = None,
-    ) -> TT["batch_and_pos_dims":..., "num_neurons", "d_model"]:
+    ) -> TT[T.batch_and_pos_dims:..., T.num_neurons, T.d_model]:
         """Returns the results of for neurons in a specific layer (ie, how much each neuron contributes to the residual stream). Does it for the subset of neurons specified by neuron_slice, defaults to all of them. Does *not* cache these because it's expensive in space and cheap to compute.
 
         Args:
@@ -364,7 +394,7 @@ class ActivationCache:
         neuron_slice: Union[Slice, SliceInput] = None,
         return_labels: bool = False,
         incl_remainder: bool = False,
-    ) -> TT["num_components", "batch_and_pos_dims":..., "d_model"]:
+    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
         """Returns a stack of all neuron results (ie residual stream contribution) up to layer L - ie the amount each individual neuron contributes to the residual stream. Also returns a list of labels of the form "L0N0" for the neurons. A good way to decompose the outputs of MLP layers into attribution by specific neurons.
 
         Note that doing this for all neurons is SUPER expensive on GPU memory and only works for small models or short inputs.
@@ -427,11 +457,11 @@ class ActivationCache:
 
     def apply_ln_to_stack(
         self,
-        residual_stack: TT["num_components", "batch_and_pos_dims":..., "d_model"],
+        residual_stack: TT[T.num_components, T.batch_and_pos_dims:..., T.d_model],
         layer: Optional[int] = None,
         mlp_input: bool = False,
         pos_slice: Union[Slice, SliceInput] = None,
-    ) -> TT["num_components", "batch_and_pos_dims":..., "d_model"]:
+    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
         """Takes a stack of components of the residual stream (eg outputs of decompose_resid or accumulated_resid), treats them as the input to a specific layer, and applies the layer norm scaling of that layer to them, using the cached scale factors - simulating what that component of the residual stream contributes to that layer's input.
 
         The layernorm scale is global across the entire residual stream for each layer, batch element and position, which is why we need to use the cached scale factors rather than just applying a new LayerNorm.
@@ -485,7 +515,7 @@ class ActivationCache:
         apply_ln: bool = False,
         pos_slice: Union[Slice, SliceInput] = None,
         return_labels: bool = False,
-    ) -> TT["num_components", "batch_and_pos_dims":..., "d_model"]:
+    ) -> TT[T.num_components, T.batch_and_pos_dims:..., T.d_model]:
         """Returns the full decomposition of the residual stream into embed, pos_embed, each head result, each neuron result, and the accumulated biases. We break down the residual stream that is input into some layer.
 
         Args:
