@@ -30,13 +30,16 @@ from transformer_lens.past_key_value_caching import (
 
 from transformer_lens.components import *
 import transformer_lens.loading_from_pretrained as loading
+from transformer_lens.torchtyping_helper import T
 import transformer_lens.utils as utils
 
-# Type alias for a single element tensor
-Loss = TT[()]
+SingleLoss = TT[()] # Type alias for a single element tensor
+LossPerToken = TT["batch", "position - 1"]
+Loss = Union[SingleLoss, LossPerToken]
+
 # Named tuple object for if we want to output both logits and loss
 class Output(NamedTuple):
-    logits: TT["batch", "pos", "d_vocab"]
+    logits: TT[T.batch, T.pos, T.d_vocab]
     loss: Loss
 
 
@@ -104,6 +107,9 @@ class HookedTransformer(HookedRootModule):
             self.pos_embed = PosEmbed(self.cfg)
             self.hook_pos_embed = HookPoint()  # [batch, pos, d__dictmodel]
 
+        if self.cfg.use_hook_tokens:
+            self.hook_tokens = HookPoint() # [batch, pos]
+
         self.blocks = nn.ModuleList(
             [
                 TransformerBlock(self.cfg, block_index)
@@ -150,7 +156,9 @@ class HookedTransformer(HookedRootModule):
         self, 
         input, 
         return_type: Literal["logits"], 
+        loss_per_token: bool = False,
         prepend_bos: bool = True,
+        stop_at_layer: Optional[int] = None, 
         past_kv_cache: Optional[HookedTransformerKeyValueCache] = None) -> Loss:
         ...
 
@@ -159,7 +167,9 @@ class HookedTransformer(HookedRootModule):
         self, 
         input, 
         return_type: Literal["loss"], 
+        loss_per_token: bool = False,
         prepend_bos: bool = True,
+        stop_at_layer: Optional[int] = None, 
         past_kv_cache: Optional[HookedTransformerKeyValueCache] = None) -> Loss:
         ...
     
@@ -168,7 +178,9 @@ class HookedTransformer(HookedRootModule):
         self, 
         input, 
         return_type: Literal["both"], 
+        loss_per_token: bool = False,
         prepend_bos: bool = True,
+        stop_at_layer: Optional[int] = None, 
         past_kv_cache: Optional[HookedTransformerKeyValueCache] = None) -> Tuple[TT["batch", "pos", "d_vocab"], Loss]:
         ...
 
@@ -177,27 +189,33 @@ class HookedTransformer(HookedRootModule):
         self, 
         input, 
         return_type: Literal[None], 
+        loss_per_token: bool = False,
         prepend_bos: bool = True,
+        stop_at_layer: Optional[int] = None, 
         past_kv_cache: Optional[HookedTransformerKeyValueCache] = None) -> None:
         ...
 
     # TODO make sure type assertions are provided
     def forward(
         self,
-        input: Union[str, List[str], TT["batch", "pos"]],
+        input: Union[str, List[str], TT[T.batch, T.pos]],
         return_type: Optional[str] = "logits",
+        loss_per_token: bool = False,
         prepend_bos: bool = True,
+        stop_at_layer: Optional[int] = None, 
         past_kv_cache: Optional[HookedTransformerKeyValueCache] = None,
     ) -> Union[
         None,
-        TT["batch", "pos", "d_vocab"],
+        TT[T.batch, T.pos, T.d_vocab],
         Loss,
-        Tuple[TT["batch", "pos", "d_vocab"], Loss],
+        Tuple[TT[T.batch, T.pos, T.d_vocab], Loss],
     ]:
         """Input is either a batch of tokens ([batch, pos]) or a text string, a string is automatically tokenized to a batch of a single element. The prepend_bos flag only applies when inputting a text string.
 
         return_type Optional[str]: The type of output to return. Can be one of: None (return nothing, don't calculate logits), 'logits' (return logits), 'loss' (return cross-entropy loss), 'both' (return logits and loss)
+        loss_per_token bool: Whether to return the (next token prediction) loss per token (True) or average (False). Average loss is a scalar (averaged over position *and* batch), per-token loss is a tensor ([batch, position-1]) - position-1 because we're predicting the next token, and there's no specified next token for the final token. Defaults to False.
         prepend_bos bool: Whether to prepend the BOS token to the input. Only applies when input is a string. Defaults to True (unlike to_tokens) - even for models not explicitly trained with this, heads often use the first position as a resting position and accordingly lose information from the first token, so this empirically seems to give better results.
+        stop_at_layer Optional[int]: If not None, stop the forward pass at the specified layer. Exclusive - ie, stop_at_layer = 0 will only run the embedding layer, stop_at_layer = 1 will run the embedding layer and the first transformer block, etc. Supports negative indexing. Useful for analysis of intermediate layers, eg finding neuron activations in layer 3 of a 24 layer model. Defaults to None (run the full model).
 
         Note that loss is the standard "predict the next token" cross-entropy loss for GPT-2 style language models - if you want a custom loss function, the recommended behaviour is returning the logits and then applying your custom loss function.
         """
@@ -240,6 +258,8 @@ class HookedTransformer(HookedRootModule):
                 cache_ctx_length == 0 or ctx_length == 1
             ), "Pass in one token at a time after loading cache"
             pos_offset = cache_ctx_length
+        if self.cfg.use_hook_tokens:
+            tokens = self.hook_tokens(tokens)
         embed = self.hook_embed(self.embed(tokens))  # [batch, pos, d_model]
         if self.cfg.positional_embedding_type == "standard":
             pos_embed = self.hook_pos_embed(
@@ -262,8 +282,15 @@ class HookedTransformer(HookedRootModule):
             raise ValueError(
                 f"Invalid positional_embedding_type passed in {self.cfg.positional_embedding_type}"
             )
-
-        for i, block in enumerate(self.blocks):
+        
+        if stop_at_layer is None:
+            # We iterate through every block by default
+            transformer_block_list = self.blocks
+        else:
+            # If we explicitly want to stop at a layer, we only iterate through the blocks up to that layer. Note that this is exclusive, eg stop_at_layer==0 means to only run the embed, stop_at_layer==-1 means to run every layer *apart* from the final one, etc.
+            transformer_block_list = self.blocks[:stop_at_layer] # type: ignore
+ 
+        for i, block in enumerate(transformer_block_list): # type: ignore
             # Note that each block includes skip connections, so we don't need
             # residual + block(residual)
             residual = block(
@@ -273,6 +300,11 @@ class HookedTransformer(HookedRootModule):
                 else None,  # Cache is contains a list of HookedTransformerKeyValueCache objects, one for each block
                 shortformer_pos_embed=shortformer_pos_embed,
             )  # [batch, pos, d_model]
+        
+        if stop_at_layer is not None:
+            # When we stop at an early layer, we end here rather than doing further computation
+            return None
+
         if self.cfg.normalization_type is not None:
             residual = self.ln_final(residual)  # [batch, pos, d_model]
         if return_type is None:
@@ -282,7 +314,7 @@ class HookedTransformer(HookedRootModule):
             if return_type == "logits":
                 return logits
             else:
-                loss = self.loss_fn(logits, tokens)
+                loss = self.loss_fn(logits, tokens, per_token=loss_per_token)
                 if return_type == "loss":
                     return loss
                 elif return_type == "both":
@@ -293,8 +325,8 @@ class HookedTransformer(HookedRootModule):
 
     def loss_fn(
         self,
-        logits: TT["batch", "pos", "d_vocab"],
-        tokens: TT["batch", "pos"],
+        logits: TT[T.batch, T.pos, T.d_vocab],
+        tokens: TT[T.batch, T.pos],
         per_token: bool = False,
     ):
         """
@@ -319,9 +351,9 @@ class HookedTransformer(HookedRootModule):
     ) -> Tuple[
         Union[
             None,
-            TT["batch", "pos", "d_vocab"],
+            TT[T.batch, T.pos, T.d_vocab],
             Loss,
-            Tuple[TT["batch", "pos", "d_vocab"], Loss],
+            Tuple[TT[T.batch, T.pos, T.d_vocab], Loss],
         ],
         Union[ActivationCache, Dict[str, torch.Tensor]],
     ]:
@@ -354,7 +386,7 @@ class HookedTransformer(HookedRootModule):
         prepend_bos: bool = True,
         move_to_device: bool = True,
         truncate: bool = True
-    ) -> TT["batch", "pos"]:
+    ) -> TT[T.batch, T.pos]:
         """
         Converts a string to a tensor of tokens. If prepend_bos is True, prepends the BOS token to the input - this is recommended when creating a sequence of tokens to be input to a model. 
 
@@ -380,14 +412,15 @@ class HookedTransformer(HookedRootModule):
             return_tensors = "pt", 
             padding = True,
             truncation = truncate,
-            max_length = self.cfg.n_ctx if truncate else None
+            max_length = self.cfg.n_ctx if truncate else None,
+            add_special_tokens = False if self.tokenizer.name_or_path.startswith('facebook/opt') else True  # As we manually add the BOS token
             )["input_ids"]
         if move_to_device:
             tokens = tokens.to(self.cfg.device)
         return tokens
 
     def to_string(
-        self, tokens: Union[TT["batch", "pos"], TT["pos"], np.ndarray, List[TT["pos"]]]
+        self, tokens: Union[TT[T.batch, T.pos], TT[T.pos], np.ndarray, List[TT[T.pos]]]
     ) -> Union[str, List[str]]:
         """
         Converts a tensor of tokens to a string (if rank 1) or a list of strings (if rank 2).
@@ -414,7 +447,7 @@ class HookedTransformer(HookedRootModule):
 
     def to_str_tokens(
         self,
-        input: Union[str, Union[TT["pos"], TT[1, "pos"]], list],
+        input: Union[str, Union[TT[T.pos], TT[1, T.pos]], list],
         prepend_bos: bool = True,
     ) -> List[str]:
         """Method to map text, a list of text or tokens to a list of tokens as strings
@@ -468,7 +501,7 @@ class HookedTransformer(HookedRootModule):
     def get_token_position(
         self,
         single_token: Union[str, int],
-        input: Union[str, Union[TT["pos"], TT[1, "pos"]]],
+        input: Union[str, Union[TT[T.pos], TT[1, T.pos]]],
         mode="first",
         prepend_bos=True,
     ):
@@ -516,7 +549,7 @@ class HookedTransformer(HookedRootModule):
         else:
             raise ValueError(f"mode must be 'first' or 'last', not {mode}")
 
-    def tokens_to_residual_directions(self, tokens: Union[str, int, TT[()], TT["position"], TT["batch", "position"]]) -> Union[TT["d_model"], TT["position", "d_model"], TT["batch", "position", "d_model"]]:
+    def tokens_to_residual_directions(self, tokens: Union[str, int, TT[()], TT[T.pos], TT[T.batch, T.pos]]) -> Union[TT[T.d_model], TT[T.pos, T.d_model], TT[T.batch, T.pos, T.d_model]]:
         """Maps tokens to a tensor with the unembedding vector for those tokens, ie the vector in the residual stream that we dot with to the get the logit for that token.
 
         WARNING: If you use this without folding in LayerNorm, the results will be misleading and may be incorrect, as the LN weights change the unembed map. This is done automatically with the fold_ln flag on from_pretrained
@@ -1071,7 +1104,7 @@ class HookedTransformer(HookedRootModule):
     @torch.inference_mode()
     def generate(
         self,
-        input: Union[str, TT["batch", "pos"]] = "",
+        input: Union[str, TT[T.batch, T.pos]] = "",
         max_new_tokens: int = 10,
         stop_at_eos: bool = True,
         eos_token_id: Optional[int] = None,
@@ -1084,7 +1117,7 @@ class HookedTransformer(HookedRootModule):
         use_past_kv_cache: bool = True,
         prepend_bos=True,
         return_type: Optional[str] = "input",
-    ) -> TT["batch", "pos + new_tokens"]:
+    ) -> TT[T.batch, T.pos_plus_new_tokens]:
         """
         Sample tokens from the model until the model outputs eos_token or max_new_tokens is reached.
 
@@ -1199,32 +1232,32 @@ class HookedTransformer(HookedRootModule):
 
     # Give access to all weights as properties.
     @property
-    def W_U(self) -> TT["d_model", "d_vocab"]:
+    def W_U(self) -> TT[T.d_model, T.d_vocab]:
         """
         Convenience to get the unembedding matrix (ie the linear map from the final residual stream to the output logits)
         """
         return self.unembed.W_U
 
     @property
-    def b_U(self) -> TT["d_vocab"]:
+    def b_U(self) -> TT[T.d_vocab]:
         return self.unembed.b_U
 
     @property
-    def W_E(self) -> TT["d_vocab", "d_model"]:
+    def W_E(self) -> TT[T.d_vocab, T.d_model]:
         """
         Convenience to get the embedding matrix
         """
         return self.embed.W_E
 
     @property
-    def W_pos(self) -> TT["n_ctx", "d_model"]:
+    def W_pos(self) -> TT[T.n_ctx, T.d_model]:
         """
         Convenience function to get the positional embedding. Only works on models with absolute positional embeddings!
         """
         return self.pos_embed.W_pos
 
     @property
-    def W_E_pos(self) -> TT["d_vocab + n_ctx", "d_model"]:
+    def W_E_pos(self) -> TT[T.d_vocab_plus_n_ctx, T.d_model]:
         """
         Concatenated W_E and W_pos. Used as a full (overcomplete) basis of the input space, useful for full QK and full OV circuits.
         """
@@ -1234,73 +1267,73 @@ class HookedTransformer(HookedRootModule):
 
     @property
     @lru_cache(maxsize=None)
-    def W_K(self) -> TT["n_layers", "n_heads", "d_model", "d_head"]:
+    def W_K(self) -> TT[T.n_layers, T.n_heads, T.d_model, T.d_head]:
         """Stacks the key weights across all layers"""
         return torch.stack([block.attn.W_K for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def W_Q(self) -> TT["n_layers", "n_heads", "d_model", "d_head"]:
+    def W_Q(self) -> TT[T.n_layers, T.n_heads, T.d_model, T.d_head]:
         """Stacks the query weights across all layers"""
         return torch.stack([block.attn.W_Q for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def W_V(self) -> TT["n_layers", "n_heads", "d_model", "d_head"]:
+    def W_V(self) -> TT[T.n_layers, T.n_heads, T.d_model, T.d_head]:
         """Stacks the value weights across all layers"""
         return torch.stack([block.attn.W_V for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def W_O(self) -> TT["n_layers", "n_heads", "d_head", "d_model"]:
+    def W_O(self) -> TT[T.n_layers, T.n_heads, T.d_head, T.d_model]:
         """Stacks the attn output weights across all layers"""
         return torch.stack([block.attn.W_O for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def W_in(self) -> TT["n_layers", "d_model", "d_mlp"]:
+    def W_in(self) -> TT[T.n_layers, T.d_model, T.d_mlp]:
         """Stacks the MLP input weights across all layers"""
         return torch.stack([block.mlp.W_in for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def W_out(self) -> TT["n_layers", "d_mlp", "d_model"]:
+    def W_out(self) -> TT[T.n_layers, T.d_mlp, T.d_model]:
         """Stacks the MLP output weights across all layers"""
         return torch.stack([block.mlp.W_out for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def b_K(self) -> TT["n_layers", "n_heads", "d_head"]:
+    def b_K(self) -> TT[T.n_layers, T.n_heads, T.d_head]:
         """Stacks the key biases across all layers"""
         return torch.stack([block.attn.b_K for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def b_Q(self) -> TT["n_layers", "n_heads", "d_head"]:
+    def b_Q(self) -> TT[T.n_layers, T.n_heads, T.d_head]:
         """Stacks the query biases across all layers"""
         return torch.stack([block.attn.b_Q for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def b_V(self) -> TT["n_layers", "n_heads", "d_head"]:
+    def b_V(self) -> TT[T.n_layers, T.n_heads, T.d_head]:
         """Stacks the value biases across all layers"""
         return torch.stack([block.attn.b_V for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def b_O(self) -> TT["n_layers", "d_model"]:
+    def b_O(self) -> TT[T.n_layers, T.d_model]:
         """Stacks the attn output biases across all layers"""
         return torch.stack([block.attn.b_O for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def b_in(self) -> TT["n_layers", "d_mlp"]:
+    def b_in(self) -> TT[T.n_layers, T.d_mlp]:
         """Stacks the MLP input biases across all layers"""
         return torch.stack([block.mlp.b_in for block in self.blocks], dim=0)
 
     @property
     @lru_cache(maxsize=None)
-    def b_out(self) -> TT["n_layers", "d_model"]:
+    def b_out(self) -> TT[T.n_layers, T.d_model]:
         """Stacks the MLP output biases across all layers"""
         return torch.stack([block.mlp.b_out for block in self.blocks], dim=0)
 
@@ -1315,7 +1348,7 @@ class HookedTransformer(HookedRootModule):
     # Various utility functions
     def accumulated_bias(
         self, layer: int, mlp_input: bool = False, include_mlp_biases=True
-    ) -> TT["layers_accumulated_over", "d_model"]:
+    ) -> TT[T.layers_accumulated_over, T.d_model]:
         """Returns the accumulated bias from all layer outputs (ie the b_Os and b_outs), up to the input of layer L.
 
         Args:
@@ -1340,7 +1373,7 @@ class HookedTransformer(HookedRootModule):
 
     def all_composition_scores(
         self, mode
-    ) -> TT["n_layers", "n_heads", "n_layers", "n_heads"]:
+    ) -> TT[T.n_layers, T.n_heads, T.n_layers, T.n_heads]:
         """Returns the Composition scores for all pairs of heads, as a L1, H1, L2, H2 tensor (which is upper triangular on the first and third axes)
 
         mode is one of ["Q", "K", "V"]
@@ -1375,37 +1408,39 @@ class HookedTransformer(HookedRootModule):
             for h in range(self.cfg.n_heads)
         ]
 
-    def load_sample_training_dataset(self):
+    def load_sample_training_dataset(self, **kwargs):
         """ 
         Helper function to load in a 10K-20K dataset of elements from the model's training data distribution. 
 
         Wrapper around utils.get_dataset, which identifies the appropriate dataset the pretrained models. Each dataset has a 'text' field, which contains the relevant info, some have several meta data fields.
 
+        Kwargs will be passed to utils.get_dataset (e.g. cache_dir to set download location)
+
         Notes:
         * GPT-2's training data is not open source. OpenWebText is a replication (links with >3 karma on Reddit)
         * OPT's training data is not open source, and is a mess of different things that is hard to replicate. I default to the Pile, which covers some of it, but imperfectly.
 
         (Some models will have actually been trained on the data supplied here, for some it's from the validation set)
         """
-        if self.cfg.original_architecture == "neel":
-            self.dataset = utils.get_dataset("c4_code")
-        elif self.cfg.original_architecture == "neel-solu-old":
-            self.dataset = utils.get_dataset("pile")
-        elif self.cfg.original_architecture == "GPT2LMHeadModel":
-            self.dataset = utils.get_dataset("openwebtext")
-        elif self.cfg.original_architecture == "GPTNeoForCausalLM":
-            self.dataset = utils.get_dataset("pile")
-        elif self.cfg.original_architecture == "GPTNeoXForCausalLM":
-            self.dataset = utils.get_dataset("pile")
-        elif self.cfg.original_architecture == "GPTJForCausalLM":
-            self.dataset = utils.get_dataset("pile")
-        elif self.cfg.original_architecture == "OPTForCausalLM":
-            self.dataset = utils.get_dataset("pile")
+        model_dataset_map = {
+            'neel': 'c4_code',
+            'neel-solu-old': 'pile',
+            'GPT2LMHeadModel': 'openwebtext',
+            'GPTNeoForCausalLM': 'pile',
+            'GPTNeoXForCausalLM': 'pile',
+            'GPTJForCausalLM': 'pile',
+            'GPTJForCausalLM': 'pile',
+            'OPTForCausalLM': 'pile',
+        }
+        if self.cfg.original_architecture in model_dataset_map:
+            self.dataset = utils.get_dataset(model_dataset_map[self.cfg.original_architecture], **kwargs)
         else:
-            raise ValueError(f"We do not have an available dataset for the relevant model: {self.cfg.original_architecture}")
+            raise ValueError(
+                f"We do not have an available dataset for the relevant model: {self.cfg.original_architecture}"
+            )
         return self.dataset
     
-    def sample_datapoint(self, tokenize=False) -> Union[str, TT[1, "pos"]]:
+    def sample_datapoint(self, tokenize=False) -> Union[str, TT[1, T.pos]]:
         """
         Helper function to randomly sample a data point from self.dataset, a small dataset from the data distribution the model was trained on. 
 
