@@ -12,6 +12,7 @@ from transformer_lens.hook_points import HookPoint
 from transformer_lens.utils import gelu_new, solu, gelu_fast
 from transformer_lens.HookedTransformerConfig import HookedTransformerConfig
 from transformer_lens.FactoredMatrix import FactoredMatrix
+from transformer_lens.torchtyping_helper import T
 
 from fancy_einsum import einsum
 
@@ -29,13 +30,13 @@ class Embed(nn.Module):
         if isinstance(cfg, Dict):
             cfg = HookedTransformerConfig.from_dict(cfg)
         self.cfg = cfg
-        self.W_E: TT["d_vocab, d_model"] = nn.Parameter(
+        self.W_E: TT[T.d_vocab, T.d_model] = nn.Parameter(
             torch.empty(self.cfg.d_vocab, self.cfg.d_model)
         )
 
     def forward(
-        self, tokens: TT["batch", "position"]
-    ) -> TT["batch", "position", "d_model"]:
+        self, tokens: TT[T.batch, T.pos]
+    ) -> TT[T.batch, T.pos, T.d_model]:
         # If A has shape [a, b] and B has shape [c, d], then A[:, B] has shape [a, c, d]
         # B acts as a tensor of indices into the second dimension (so >=0 and <b)
         return self.W_E[tokens, :]
@@ -48,14 +49,14 @@ class Unembed(nn.Module):
             cfg = HookedTransformerConfig.from_dict(cfg)
         self.cfg = cfg
         # Note that there's a separate variable for d_vocab_out and d_vocab (the input vocab size). For language tasks these are always the same, but for algorithmic tasks we may want them to be different.
-        self.W_U: TT["d_model", "d_vocab_out"] = nn.Parameter(
+        self.W_U: TT[T.d_model, T.d_vocab_out] = nn.Parameter(
             torch.empty(self.cfg.d_model, self.cfg.d_vocab_out)
         )
-        self.b_U: TT["d_vocab_out"] = nn.Parameter(torch.zeros(self.cfg.d_vocab_out))
+        self.b_U: TT[T.d_vocab_out] = nn.Parameter(torch.zeros(self.cfg.d_vocab_out))
 
     def forward(
-        self, residual: TT["batch", "position", "d_model"]
-    ) -> TT["batch", "position", "d_vocab_out"]:
+        self, residual: TT[T.batch, T.pos, T.d_model]
+    ) -> TT[T.batch, T.pos, T.d_vocab_out]:
         return (
             einsum(
                 "batch pos d_model, d_model vocab -> batch pos vocab",
@@ -76,8 +77,8 @@ class PosEmbed(nn.Module):
         self.W_pos = nn.Parameter(torch.empty(self.cfg.n_ctx, self.cfg.d_model))
 
     def forward(
-        self, tokens: TT["batch", "position"], past_kv_pos_offset: int = 0
-    ) -> TT["batch", "position", "d_model"]:
+        self, tokens: TT[T.batch, T.pos], past_kv_pos_offset: int = 0
+    ) -> TT[T.batch, T.pos, T.d_model]:
         """Tokens have shape [batch, pos]
         past_kv_pos_offset is the length of tokens in the past_kv_cache (if used, defaults to zero if unused)
         Output shape [pos, d_model] - will be broadcast along batch dim"""
@@ -115,10 +116,10 @@ class LayerNormPre(nn.Module):
         self.hook_normalized = HookPoint()  # [batch, pos, length]
 
     def forward(
-        self, x: TT["batch", "position", "length"]
-    ) -> TT["batch", "position", "length"]:
+        self, x: TT[T.batch, T.pos, T.length]
+    ) -> TT[T.batch, T.pos, T.length]:
         x = x - x.mean(axis=-1, keepdim=True)  # [batch, pos, length]
-        scale: TT["batch", "position", 1] = self.hook_scale(
+        scale: TT[T.batch, T.pos, 1] = self.hook_scale(
             (x.pow(2).mean(-1, keepdim=True) + self.eps).sqrt()
         )
         return self.hook_normalized(x / scale)
@@ -153,10 +154,10 @@ class LayerNorm(nn.Module):
         self.hook_normalized = HookPoint()  # [batch, pos, length]
 
     def forward(
-        self, x: TT["batch", "position", "length"]
-    ) -> TT["batch", "position", "length"]:
+        self, x: TT[T.batch, T.pos, T.length]
+    ) -> TT[T.batch, T.pos, T.length]:
         x = x - x.mean(axis=-1, keepdim=True)  # [batch, pos, length]
-        scale: TT["batch", "position", 1] = self.hook_scale(
+        scale: TT[T.batch, T.pos, 1] = self.hook_scale(
             (x.pow(2).mean(-1, keepdim=True) + self.eps).sqrt()
         )
         x = x / scale  # [batch, pos, length]
@@ -177,9 +178,9 @@ class RMSNormPre(nn.Module):
         self.hook_normalized = HookPoint()  # [batch, pos, length]
 
     def forward(
-        self, x: TT["batch", "position", "length"]
-    ) -> TT["batch", "position", "length"]:
-        scale: TT["batch", "position", 1] = self.hook_scale(
+        self, x: TT[T.batch, T.pos, T.length]
+    ) -> TT[T.batch, T.pos, T.length]:
+        scale: TT[T.batch, T.pos, 1] = self.hook_scale(
             (x.pow(2).mean(-1, keepdim=True) + self.eps).sqrt()
         )
         return self.hook_normalized(x / scale)  # [batch, pos, length]
@@ -212,9 +213,9 @@ class RMSNorm(nn.Module):
         self.hook_normalized = HookPoint()  # [batch, pos, length]
 
     def forward(
-        self, x: TT["batch", "position", "length"]
-    ) -> TT["batch", "position", "length"]:
-        scale: TT["batch", "position", 1] = self.hook_scale(
+        self, x: TT[T.batch, T.pos, T.length]
+    ) -> TT[T.batch, T.pos, T.length]:
+        scale: TT[T.batch, T.pos, 1] = self.hook_scale(
             (x.pow(2).mean(-1, keepdim=True) + self.eps).sqrt()
         )
         x = self.hook_normalized(x / scale)  # [batch, pos, length]
@@ -341,10 +342,10 @@ class Attention(nn.Module):
 
     def forward(
         self,
-        resid_pre: TT["batch", "pos", "d_model"],
-        shortformer_pos_embed: Optional[TT["batch", "pos", "d_model"]] = None,
+        resid_pre: TT[T.batch, T.pos, T.d_model],
+        shortformer_pos_embed: Optional[TT[T.batch, T.pos, T.d_model]] = None,
         past_kv_cache_entry: Optional[HookedTransformerKeyValueCacheEntry] = None,
-    ) -> TT["batch", "pos", "d_model"]:
+    ) -> TT[T.batch, T.pos, T.d_model]:
         """
         shortformer_pos_embed is only used if self.cfg.positional_embedding_type == "shortformer", else defaults to None and is irrelevant. See HookedTransformerConfig for more details
         past_kv_cache_entry is an optional entry of past keys and values for this layer, only relevant if generating text. Defaults to None
@@ -457,7 +458,7 @@ class Attention(nn.Module):
 
     def apply_causal_mask(
         self,
-        attn_scores: TT["batch", "head_index", "pos", "pos + past_kv_pos_offset"],
+        attn_scores: TT[T.batch, T.head_index, T.pos, T.pos_plus_past_kv_pos_offset],
         past_kv_pos_offset: int = 0,
     ):
         # The query context length is the number of positions we take queries from - if not using a past_kv_cache this is just the context length (for the current prompt), but if we're caching it's just a single token.
@@ -480,11 +481,11 @@ class Attention(nn.Module):
 
     def shortformer_calculate_qk(
         self,
-        x: TT["batch", "pos", "d_model"],
-        shortformer_pos_embed: TT["batch", "pos", "d_model"],
+        x: TT[T.batch, T.pos, T.d_model],
+        shortformer_pos_embed: TT[T.batch, T.pos, T.d_model],
     ) -> Tuple[
-        TT["batch", "pos", "head_index", "d_head"],
-        TT["batch", "pos", "head_index", "d_head"],
+        TT[T.batch, T.pos, T.head_index, T.d_head],
+        TT[T.batch, T.pos, T.head_index, T.d_head],
     ]:
         # We add on the positional encodings to the residual stream JUST for the keys and queries, it's not added to the normal residual stream.
         attn_input = self.hook_attn_input(
@@ -512,12 +513,12 @@ class Attention(nn.Module):
 
     def rotary_rotate_qk(
         self,
-        q: TT["batch", "pos", "head_index", "d_head"],
-        k: TT["batch", "pos", "head_index", "d_head"],
+        q: TT[T.batch, T.pos, T.head_index, T.d_head],
+        k: TT[T.batch, T.pos, T.head_index, T.d_head],
         past_kv_pos_offset,
     ) -> Tuple[
-        TT["batch", "pos", "head_index", "d_head"],
-        TT["batch", "pos", "head_index", "d_head"],
+        TT[T.batch, T.pos, T.head_index, T.d_head],
+        TT[T.batch, T.pos, T.head_index, T.d_head],
     ]:
         # We first apply standard q and k calculation
         q = self.hook_rot_q(self.apply_rotary(q, past_kv_pos_offset))
@@ -526,7 +527,7 @@ class Attention(nn.Module):
 
     def calculate_sin_cos_rotary(
         self, rotary_dim: int, n_ctx: int, base: int = 10000
-    ) -> Tuple[TT["n_ctx", "rotary_dim"], TT["n_ctx", "rotary_dim"]]:
+    ) -> Tuple[TT[T.n_ctx, T.rotary_dim], TT[T.n_ctx, T.rotary_dim]]:
         """
         Calculate the sine and cosine waves to use in a rotary embedding. See https://blog.eleuther.ai/rotary-embeddings/ for details
 
@@ -545,7 +546,7 @@ class Attention(nn.Module):
         angles = pos[:, None] / freq[None, :]
         return torch.sin(angles), torch.cos(angles)
 
-    def rotate_every_two(self, x: TT[..., "rotary_dim"]) -> TT[..., "rotary_dim"]:
+    def rotate_every_two(self, x: TT[..., T.rotary_dim]) -> TT[..., T.rotary_dim]:
         """
         Rotary helper function, splits x into blocks of size 2 along the final axis and maps [x0, x1] to [-x1, x0]
 
@@ -565,8 +566,8 @@ class Attention(nn.Module):
         return rot_x
 
     def apply_rotary(
-        self, x: TT["batch", "pos", "head_index", "d_head"], past_kv_pos_offset=0
-    ) -> TT["batch", "pos", "head_index", "d_head"]:
+        self, x: TT[T.batch, T.pos, T.head_index, T.d_head], past_kv_pos_offset=0
+    ) -> TT[T.batch, T.pos, T.head_index, T.d_head]:
         # Only apply rotary to first rotary_dim dimensions (eg, if rotary_dim=64 and d_head=256, only apply to first 1/4 of dimensions)
         x_pos = x.size(1)
         x_rot = x[..., : self.cfg.rotary_dim]
@@ -619,8 +620,8 @@ class MLP(nn.Module):
             raise ValueError(f"Invalid activation function name: {self.cfg.act_fn}")
 
     def forward(
-        self, x: TT["batch", "pos", "d_model"]
-    ) -> TT["batch", "pos", "d_model"]:
+        self, x: TT[T.batch, T.pos, T.d_model]
+    ) -> TT[T.batch, T.pos, T.d_model]:
         # Technically, all these einsums could be done with a single matmul, but this is more readable.
         pre_act = self.hook_pre(
             einsum("batch pos d_model, d_model d_mlp -> batch pos d_mlp", x, self.W_in)
@@ -684,10 +685,10 @@ class TransformerBlock(nn.Module):
 
     def forward(
         self,
-        resid_pre: TT["batch", "position", "d_model"],
-        shortformer_pos_embed: Optional[TT["batch", "position", "d_model"]] = None,
+        resid_pre: TT[T.batch, T.pos, T.d_model],
+        shortformer_pos_embed: Optional[TT[T.batch, T.pos, T.d_model]] = None,
         past_kv_cache_entry: Optional[HookedTransformerKeyValueCacheEntry] = None,
-    ) -> TT["batch", "position", "d_model"]:
+    ) -> TT[T.batch, T.pos, T.d_model]:
         """A single Transformer block.
 
         Args:
