@@ -15,6 +15,7 @@ from datasets.arrow_dataset import Dataset
 from datasets.load import load_dataset
 
 from transformer_lens import FactoredMatrix
+from transformer_lens.torchtyping_helper import T
 
 CACHE_DIR = transformers.TRANSFORMERS_CACHE
 import json
@@ -73,10 +74,10 @@ def to_numpy(tensor):
 
 
 def lm_cross_entropy_loss(
-    logits: TT["batch", "pos", "d_vocab"],
-    tokens: TT["batch", "pos"],
+    logits: TT[T.batch, T.pos, T.d_vocab],
+    tokens: TT[T.batch, T.pos],
     per_token: bool = False,
-) -> Union[TT[()], TT["batch", "pos"]]:
+) -> Union[TT[()], TT[T.batch, T.pos]]:
     """Cross entropy loss for the language model, gives the loss for predicting the NEXT token.
 
     Args:
@@ -98,10 +99,10 @@ def lm_cross_entropy_loss(
 
 
 def lm_accuracy(
-    logits: TT["batch", "pos", "d_vocab"],
-    tokens: TT["batch", "pos"],
+    logits: TT[T.batch, T.pos, T.d_vocab],
+    tokens: TT[T.batch, T.pos],
     per_token: bool = False,
-) -> Union[TT[()], TT["batch", "pos"]]:
+) -> Union[TT[()], TT[T.batch, T.pos]]:
     """Cross-Entropy Accuracy for Language Modelling. We measure the accuracy on the logits for predicting the NEXT token.
 
     If per_token is True, returns the boolean for top 1 accuracy for each token in the batch. Note that this has size [batch, seq_len-1], as we cannot predict the first token.
@@ -114,7 +115,7 @@ def lm_accuracy(
         return correct_matches.sum() / correct_matches.numel()
 
 
-def gelu_new(input: TT["batch", "pos", "d_mlp"]) -> TT["batch", "pos", "d_mlp"]:
+def gelu_new(input: TT[T.batch, T.pos, T.d_mlp]) -> TT[T.batch, T.pos, T.d_mlp]:
     # Implementation of GeLU used by GPT2 - subtly different from PyTorch's
     return (
         0.5
@@ -128,7 +129,7 @@ def gelu_new(input: TT["batch", "pos", "d_mlp"]) -> TT["batch", "pos", "d_mlp"]:
     )
 
 
-def gelu_fast(input: TT["batch", "pos", "d_mlp"]) -> TT["batch", "pos", "d_mlp"]:
+def gelu_fast(input: TT[T.batch, T.pos, T.d_mlp]) -> TT[T.batch, T.pos, T.d_mlp]:
     return (
         0.5
         * input
@@ -136,7 +137,7 @@ def gelu_fast(input: TT["batch", "pos", "d_mlp"]) -> TT["batch", "pos", "d_mlp"]
     )
 
 
-def solu(input: TT["batch", "pos", "d_mlp"]) -> TT["batch", "pos", "d_mlp"]:
+def solu(input: TT[T.batch, T.pos, T.d_mlp]) -> TT[T.batch, T.pos, T.d_mlp]:
     """
     SoLU activation function as described by
     https://transformer-circuits.pub/2022/solu/index.html.
@@ -241,13 +242,13 @@ tokenize_and_concatenate(data, tokenizer, streaming=False, column_name="text")
 """
 
 def sample_logits(
-    final_logits: TT["batch", "d_vocab"],
+    final_logits: TT[T.batch, T.d_vocab],
     top_k: Optional[int] = None,
     top_p: Optional[float] = None,
     temperature: float = 1.0,
     freq_penalty: float = 0.0,
-    tokens: Optional[TT["batch", "pos"]] = None,
-) -> TT["batch"]:
+    tokens: Optional[TT[T.batch, T.pos]] = None,
+) -> TT[T.batch]:
     """
     Sample from the logits, in order to generate text
 
@@ -513,7 +514,7 @@ def test_prompt(
 
 
 # %%
-def transpose(tensor: TT[..., "a", "b"]) -> TT[..., "b", "a"]:
+def transpose(tensor: TT[..., T.a, T.b]) -> TT[..., T.b, T.a]:
     """
     Utility to swap the last two dimensions of a tensor, regardless of the number of leading dimensions
     """
@@ -522,7 +523,7 @@ def transpose(tensor: TT[..., "a", "b"]) -> TT[..., "b", "a"]:
 def composition_scores(
     left: FactoredMatrix, right: FactoredMatrix, broadcast_dims=True
 ) -> Union[
-    TT["leading_dims":...], TT["leading_dims_left":..., "leading_dims_right":...]
+    TT[T.leading_dims:...], TT[T.leading_dims_left:..., T.leading_dims_right:...]
 ]:
     """
     See `HookedTransformer.all_composition_scores` for documentation.
@@ -547,12 +548,14 @@ def composition_scores(
 
 
 # %%
-def get_dataset(dataset_name: str) -> Dataset:
+def get_dataset(dataset_name: str, **kwargs) -> Dataset:
     """
     Returns a small HuggingFace dataset, for easy testing and exploration. Accesses several convenience datasets with 10,000 elements (dealing with the enormous 100GB - 2TB datasets is a lot of effort!). Note that it returns a dataset (ie a dictionary containing all the data), *not* a DataLoader (iterator over the data + some fancy features). But you can easily convert it to a DataLoader. 
     
     Each dataset has a 'text' field, which contains the relevant info, some also have several meta data fields
 
+    Kwargs will be passed to the huggingface dataset loading function, e.g. "data_dir"
+    
     Possible inputs:
     * openwebtext (approx the GPT-2 training data https://huggingface.co/datasets/openwebtext)
     * pile (The Pile, a big mess of tons of diverse data https://pile.eleuther.ai/)
@@ -561,19 +564,19 @@ def get_dataset(dataset_name: str) -> Dataset:
     * c4_code (c4 + code - the 20K data points from c4-10k and code-10k. This is the mix of datasets used to train my interpretability-friendly models, though note that they are *not* in the correct ratio! There's 10K texts for each, but about 22M tokens of code and 5M tokens of C4)
     * wiki (Wikipedia, generated from the 20220301.en split of https://huggingface.co/datasets/wikipedia )
     """
-    if dataset_name in ["openwebtext", "owt"]:
-        dataset = load_dataset("stas/openwebtext-10k", split='train')
-    elif dataset_name == "pile":
-        dataset = load_dataset("NeelNanda/pile-10k", split='train')
-    elif dataset_name == "c4":
-        dataset = load_dataset("NeelNanda/c4-10k", split='train')
-    elif dataset_name in ["code", "python"]:
-        dataset = load_dataset("NeelNanda/code-10k", split='train')
-    elif dataset_name in ["c4_code", "c4-code"]:
-        # Note that this one has 20K 
-        dataset = load_dataset("NeelNanda/c4-code-20k", split='train')
-    elif dataset_name == "wiki":
-        dataset = load_dataset("NeelNanda/wiki-10k", split="train")
+    dataset_aliases = {
+        'openwebtext': 'stas/openwebtext-10k',
+        'owt': 'stas/openwebtext-10k',
+        'pile': 'NeelNanda/pile-10k',
+        'c4': 'NeelNanda/c4-10k',
+        'code': 'NeelNanda/code-10k',
+        'python': 'NeelNanda/code-10k',
+        'c4_code': 'NeelNanda/c4-code-20k',
+        'c4-code': 'NeelNanda/c4-code-20k',
+        'wiki': 'NeelNanda/wiki-10k'
+    }
+    if dataset_name in dataset_aliases:
+        dataset = load_dataset(dataset_aliases[dataset_name], split='train', **kwargs)
     else:
         raise ValueError(f"Dataset {dataset_name} not supported")
-    return dataset
\ No newline at end of file
+    return dataset
